function [y,fax,dfax,taumax]=osdmorlet(xtype,x,tax,nf,sigma,tau,faxtype,om1)
%   [y,fax,dfax,taumax]=osdmorlet(xtype,x,tax,nf,sigma,tau,faxtype,om1) returns the forward
%   or inverse one-sided discrete Morlet transform of order nf for tax
%   with regular sampling (integer values) not necessarily uniform
%   using window parameters sigma and tau

if (nargin < 4) || (nargin > 8)
    help osdmorlet, return
end
if  (length(xtype) ~= 1) || (abs(xtype) ~= 1)
    error('xtype must be 1 or -1')
end
if nargin < 5
    sigma = inf;    % flat window
end
if nargin < 6
    tau = 0;        % unit impulse
end
if nargin < 7
    faxtype = 0;    % fax parity
end
if nargin < 8
    om1 = 1;        % frequency offset
end

tax = tax(:);           % tax is now a column vector
nd = length(tax);       % number of data entries
tax = tax-tax(1)+1;     % assign 1 to first entry
nt = tax(end);          % duration of data

if mod(faxtype,2)
    fax = [0:nf]/2/nf;                  % frequency axis
    dfax = 1/2/nf*ones(1,nf+1);         % integration measure
    dfax([1 end]) = dfax([1 end])/2;    % edge pixels
else
    fax = ([1:nf]-1/2)/2/nf;            % frequency axis
    dfax = 1/2/nf*ones(1,nf);           % integration measure
end
nff = nf+mod(faxtype,2);                % number of pixels
flo = fax(1+mod(faxtype,2));            % lowest nonzero frequency
taumax = ceil(tau/flo);                 % maximum window size
tmax = nt+2*taumax;                     % transform duration

if xtype == 1   % forward transform
    xgap = zeros(1,nt); % zero pad gaps
    xgap(tax) = x(:);   % fill in data
    y = zeros(nff,tmax);
    for kf = 1:nff
        f = fax(kf); 
        [tauf,psi,phi,tw] = makepsi(f,sigma,tau,taumax,om1);
%plot(tw,real(psi),tw,imag(psi)), pause(.05)
        rowf = conv(psi,xgap);
%        rowf = conv(phi,exp(-i*2*pi*f*om1*[1:nt]).*xgap);
        y(kf,taumax+[1-tauf:nt+tauf]) = rowf;
    end
elseif xtype == -1  % inverse transform
    faxtax = zeros(nff,nt);
    for kf = 1:nff
        f = fax(kf); 
        [tauf,psi,phi] = makepsi(f,sigma,tau,taumax,om1);
        rowf = conv(psi,x(kf,taumax+[1-tauf:nt+tauf]));
%        rowf = exp(i*2*pi*f*om1*[1-tauf*2:nt+tauf*2]).*conv(phi,x(kf,taumax+[1-tauf:nt+tauf]));
        faxtax(kf,:) = rowf(tauf*2+[1:nt]);
    end
    y = real(dfax*faxtax(:,tax));
end % if xtype

return
%%%%%%

function [tauf,psi,phi,tw] = makepsi(f,sigma,tau,taumax,om1)
%   make basis function

sigmaf = sigma/f;                   % window decay rel t
tauf = min([ceil(tau/f) taumax]);   % window duration
%sigmaf = sigma/tau*tauf;            % window decay rel tau
tw = -tauf:tauf;                    % window time axis
phi = exp(-pi*tw.^2/sigmaf^2/2);    % window amplitude
phi = phi/sqrt(phi*phi'/2);         % normalized to unit energy x 2
theta = exp(i*2*pi*f*om1*tw);       % window phase
psi = phi.*theta;                   % basis function at f

return
%%%%%%

